package com.jumper.record.heartrate.web;
import java.io.BufferedReader;
import java.io.IOException;
import java.io.PrintWriter;
import java.util.ArrayList;
import java.util.List;
import javax.servlet.ServletException;
import javax.servlet.http.HttpServlet;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import org.apache.commons.lang3.StringUtils;
import org.apache.log4j.Logger;
import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.jumper.record.common.entity.ReturnMsg;
import com.jumper.record.heartrate.entity.HeartrateRecord;
import com.jumper.record.heartrate.entity.RealtimeUploadHeartrateReq;
import com.jumper.record.utils.Const;

/**
 * yangsheng@angeldoctor.net
 * Servlet 实时胎心监测
 */
public class RealtimeHeartrate extends HttpServlet {
	private static final long serialVersionUID = 1L;
	protected Logger logger = Logger.getLogger(this.getClass());   
    /**
     * @see HttpServlet#HttpServlet()
     */
    public RealtimeHeartrate() {
        super();
    }

	/**
	 * 获取监测的数据
	 */
	@SuppressWarnings({ "rawtypes", "unchecked" })
	protected void doGet(HttpServletRequest request, HttpServletResponse response) throws ServletException, IOException {
		ReturnMsg ret = new ReturnMsg();
		ret.setMsg(Const.SUCCESS);
		ret.setMsgbox("获取所有实时胎心测量数据成功!");
		ret.setData(Const.realTimeHeartrateData);
		response.setCharacterEncoding("utf-8");
		response.setContentType("application/json");
		response.setHeader("Access-Control-Allow-Origin", "*");
		response.setHeader("Access-Control-Allow-Methods","GET");  
		response.setHeader("Access-Control-Allow-Headers","x-requested-with,content-type");  
		PrintWriter pw = response.getWriter();
		pw.println(JSON.toJSONString(ret));
		pw.close();
	}

	/**
	 *
	 *接收手机端推动的数据
	 */
	@SuppressWarnings("rawtypes")
	protected void doPost(HttpServletRequest request, HttpServletResponse response) throws ServletException, IOException {
		request.setCharacterEncoding("utf-8");
		BufferedReader reader = request.getReader();
		String line = null;
        StringBuilder sb = new StringBuilder();
        while((line = reader.readLine())!=null){
            sb.append(line);
        }
        reader.close();
        JSONObject realTimeJSON  = JSON.parseObject(sb.toString());
        String dataType = realTimeJSON.getString("dataType");
        ReturnMsg ret =null;
        if(dataType.equals("heartRate")){
        	 RealtimeUploadHeartrateReq realTimeHeartrateRecordReq = realTimeJSON.toJavaObject(RealtimeUploadHeartrateReq.class);
        	 ret = this.uploadRealTimeFetalrate(realTimeHeartrateRecordReq);
        }
        response.setCharacterEncoding("utf-8");
		response.setContentType("application/json");
		PrintWriter pw = response.getWriter();
		pw.println(JSON.toJSONString(ret));
		pw.close();
	}
	
	
	/**
	 * 
	 *实时胎心测量开始
	 * @author yangsheng@angeldoctor
	 * @param realTimeHeartrateRecordReq
	 * @return
	 */
	@SuppressWarnings("rawtypes")
	public ReturnMsg uploadRealTimeFetalrate(RealtimeUploadHeartrateReq realTimeHeartrateRecordReq) {
		ReturnMsg ret = new ReturnMsg();
		if(StringUtils.isBlank(realTimeHeartrateRecordReq.getDeviceMac())){
			ret.setMsg(Const.FAILED);
			ret.setMsgbox("设备mac地址不能不能为空!");
			return ret;
		}
		if(StringUtils.isBlank(realTimeHeartrateRecordReq.getOperatorType())){
			ret.setMsg(Const.FAILED);
			ret.setMsgbox("操作类型不能为空!");
			return ret;
		}
		try {
			String operatorType =realTimeHeartrateRecordReq.getOperatorType();
			switch(operatorType){
				case "start":
					 this.start(realTimeHeartrateRecordReq, ret);
					 return ret;
				case "add":
					this.add(realTimeHeartrateRecordReq.getData(), ret);
					return ret;
				case "stop":
					this.stop(realTimeHeartrateRecordReq.getData(), ret);
					return ret;
				case "reset":
					this.reset(realTimeHeartrateRecordReq.getData(), ret);
					return ret;
				default:
					ret.setMsg(Const.FAILED);
					ret.setMsgbox("操作类型无效请检查!");
					return ret;
			}
		} catch (Exception e) {
			logger.error("RealtimeHeartrate.uploadRealTimeFetalrate.error",e);
			ret.setMsg(Const.FAILED);
			ret.setMsgbox("添加实时监测用户异常!");
			return ret;
		}
	}
	
	/**
	 * 
	 *获取所有实时胎心测量数据
	 * @author yangsheng@angeldoctor
	 * @return
	 */
	@SuppressWarnings({ "rawtypes", "unchecked" })
	public ReturnMsg clearAllRealTimeFetalrateData() {
		Const.realTimeHeartrateData.clear();
		ReturnMsg ret = new ReturnMsg();
		ret.setMsg(Const.SUCCESS);
		ret.setMsgbox("获取所有实时胎心测量数据成功!");
		ret.setData(Const.realTimeHeartrateData);
		return ret;
	}
	
	@SuppressWarnings("rawtypes")
	private void start(RealtimeUploadHeartrateReq realTimeHeartrateRecordReq,ReturnMsg ret){
		if(!Const.realTimeHeartrateData.containsKey("user")){
			JSONObject userJSON = new JSONObject(true);
			userJSON.put("userId", realTimeHeartrateRecordReq.getOpenUserId());
			userJSON.put("deviceMac", realTimeHeartrateRecordReq.getDeviceMac());
			userJSON.put("seriesNumber", realTimeHeartrateRecordReq.getSeriesNumber());
			
			JSONObject recordJSON = new JSONObject(true);
			recordJSON.put("fetalMove",new ArrayList<Integer>());
			recordJSON.put("heartRate", new ArrayList<Integer>());
			recordJSON.put("uterus", new ArrayList<Integer>());
			
			Const.realTimeHeartrateData.put("user", userJSON);
			Const.realTimeHeartrateData.put("record", recordJSON);
		}else{
			JSONObject recordJSON = Const.realTimeHeartrateData.getJSONObject("record");
			recordJSON.put("fetalMove",new ArrayList<Integer>());
			recordJSON.put("heartRate", new ArrayList<Integer>());
			recordJSON.put("uterus", new ArrayList<Integer>());
		}
		ret.setMsg(Const.SUCCESS);
		ret.setMsgbox("初始化完成!");
	}
	
	@SuppressWarnings({ "rawtypes", "unchecked" })
	private void add(HeartrateRecord realTimeHeartrateRecordReq,ReturnMsg ret){
		//监测对象
		if(!Const.realTimeHeartrateData.containsKey("record")){
			ret.setMsg(Const.FAILED);
			ret.setMsgbox("初始化接口未调用!");
			return;
		}
		JSONObject recordJSON = Const.realTimeHeartrateData.getJSONObject("record");
		if(StringUtils.isNotBlank(realTimeHeartrateRecordReq.getFetalMove())){
			List<Integer> fetalMove = (List<Integer>) recordJSON.get("fetalMove");
			if(StringUtils.isNotBlank(realTimeHeartrateRecordReq.getFetalMove())){
				String tempTime [] = realTimeHeartrateRecordReq.getFetalMove().split(":");
				int second = ( Integer.parseInt(tempTime[0])*60) + Integer.parseInt(tempTime[1]);
				fetalMove.add(second);
			}
			recordJSON.put("fetalMove",fetalMove);
		}
		if(realTimeHeartrateRecordReq.getHeartRate() !=null && realTimeHeartrateRecordReq.getHeartRate().length>0){
			List<Integer> heartRate =  (List<Integer>) recordJSON.get("heartRate");
			Integer tempHeartRate [] = realTimeHeartrateRecordReq.getHeartRate();
			for(Integer hr :tempHeartRate){
				heartRate.add(hr);
			}
			recordJSON.put("heartRate",heartRate);
		}
		if(realTimeHeartrateRecordReq.getUterus() !=null && realTimeHeartrateRecordReq.getUterus().length >0){
			List<Integer> uterus = (List<Integer>) recordJSON.get("uterus");
			Integer tempUterus[] = realTimeHeartrateRecordReq.getUterus();
			for(Integer ut : tempUterus){
				uterus.add(ut);
			}
			recordJSON.put("uterus",uterus);
		}
		ret.setMsg(Const.SUCCESS);
		ret.setMsgbox("添加胎心成功!");
	}
	
	
	@SuppressWarnings({ "rawtypes" })
	private void stop(HeartrateRecord realTimeHeartrateRecordReq,ReturnMsg ret){
		if(!Const.realTimeHeartrateData.containsKey("record")){
			ret.setMsg(Const.FAILED);
			ret.setMsgbox("非法请求!");
			return;
		}
		Const.realTimeHeartrateData = new JSONObject();
		ret.setMsg(Const.SUCCESS);
		ret.setMsgbox("清楚数据完成!");
	}
	
	@SuppressWarnings("rawtypes")
	public void reset(HeartrateRecord realTimeHeartrateRecordReq,ReturnMsg ret){
		if(!Const.realTimeHeartrateData.containsKey("record")){
			ret.setMsg(Const.FAILED);
			ret.setMsgbox("非法请求!");
			return;
		}
		JSONObject recordJSON = Const.realTimeHeartrateData.getJSONObject("record");
		recordJSON.put("fetalMove",new ArrayList<Integer>());
		recordJSON.put("heartRate",new ArrayList<Integer>());
		recordJSON.put("uterus",new ArrayList<Integer>());
		ret.setMsg(Const.SUCCESS);
		ret.setMsgbox("重置数据完成!");
	}
}
