/** 
 * Project Name:jumper-test 
 * File Name:HttpClientTools.java 
 * Package Name:com.jumper.record.utils 
 * Date:2017年3月21日上午11:00:36 
 * Copyright (c) 2017, yangsheng@angeldoctor All Rights Reserved. 
 * 
*/  
package com.jumper.record.utils;
import java.io.File;
import java.io.IOException;
import java.io.InputStream;
import java.io.UnsupportedEncodingException;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import org.apache.commons.lang3.StringUtils;
import org.apache.http.HttpEntity;
import org.apache.http.client.ClientProtocolException;
import org.apache.http.client.config.RequestConfig;
import org.apache.http.client.entity.UrlEncodedFormEntity;
import org.apache.http.client.methods.CloseableHttpResponse;
import org.apache.http.client.methods.HttpGet;
import org.apache.http.client.methods.HttpPost;
import org.apache.http.client.methods.HttpRequestBase;
import org.apache.http.entity.ContentType;
import org.apache.http.entity.StringEntity;
import org.apache.http.entity.mime.MultipartEntityBuilder;
import org.apache.http.entity.mime.content.FileBody;
import org.apache.http.impl.client.CloseableHttpClient;
import org.apache.http.impl.client.HttpClients;
import org.apache.http.util.EntityUtils;
import com.alibaba.fastjson.JSONObject;
import lombok.Builder;
import lombok.Getter;
import lombok.extern.log4j.Log4j;
import org.apache.http.NameValuePair;
import org.apache.http.message.BasicNameValuePair;
@Log4j
@Builder
public class HttpClientTools<T> {
	/**
	 * 请求URl
	 */
	private String url;
	/**
	 * 从连接池获取连接的timeout
	 */
	private int connectionRequestTimeout=5000;
	/**
	 * 和服务器建立连接的timeout
	 * 连接某个URL等待超时时间 单位毫秒
	 */
	private int connectTimeout=1000*30;
	/**
	 * 从服务器读取数据的timeout
	 */
	private int socketTimeout=1000*60;
	/**
	 * 请求体body数据
	 */
	private JSONObject json;
	/**
	 * form表单提交
	 */
	private  Map<String,T> paramMap;
	/**
	 * 多文件上传
	 */
	private List<File> fileList;
	/**
	 * 单文件上传
	 */
	private File file;
	/**
	 * 多个流文件上传, key 文件全名  value 文件流
	 */
	private  Map<String,InputStream> isMap;
	/**
	 * 请求结果
	 */
	@Getter
	private String result = "";
	/**
	 * 发送http get 请求
	 * @author yangsheng@angeldoctor 
	 * @return
	 */
	public String get(){
		if(StringUtils.isBlank(url)){
			throw new NullPointerException("url is null");
		}
		HttpGet httpGet = new HttpGet(url);
		httpGet.setConfig(this.getReqConfig());
		return this.execute(httpGet);
	}
	
	/**
	 * 发送 json 格式post请求
	 * @author yangsheng@angeldoctor 
	 * @return
	 */
	public  String postJSON(){
		if(json ==null){
			throw new NullPointerException("json is null");
		}
		HttpPost httpPost = this.getPost();
		StringEntity stringEntity = new StringEntity(json.toJSONString(), "UTF-8");
		httpPost.setEntity(stringEntity);
		httpPost.setHeader("Content-type", "application/json");
		return this.execute(httpPost);
	}
	
	/**
	 * 发送form表单请求
	 * @author yangsheng@angeldoctor 
	 * @return
	 */
	public String postForm(){
		if(paramMap ==null || paramMap.size() <=0){
			throw new NullPointerException("paramMap is null or is empty");
		}
		HttpPost httpPost = this.getPost();
		List <NameValuePair> nvps = new ArrayList <NameValuePair>(paramMap.size());
		for(Map.Entry<String,T> entry :paramMap.entrySet()){
			log.info(entry.getKey()+":"+entry.getValue());
    	  nvps.add(new BasicNameValuePair(entry.getKey(), entry.getValue().toString()));
		}
		try {
			httpPost.setEntity(new UrlEncodedFormEntity(nvps,"UTF-8"));
		} catch (UnsupportedEncodingException e) {
			log.error("HttpClientTools.postNormalForm.error",e);
		}
		return this.execute(httpPost);
	}
	
	/**
	 * 上传文件到基础服务
	 * @author yangsheng@angeldoctor 
	 * @return
	 */
	public String uploadMultiInputStreamToCommentService(){
		if(isMap ==null || isMap.size() <=0){
			throw new NullPointerException("file Map is null");
		}
		HttpPost httpPost = this.getPost();
		MultipartEntityBuilder builer = MultipartEntityBuilder.create();
		for(Map.Entry<String,InputStream> entry :isMap.entrySet()){
			builer.addBinaryBody("file", entry.getValue(), ContentType.create("application/octet-stream"),entry.getKey());
		}
		HttpEntity reqEntity =builer.build();
		httpPost.addHeader(reqEntity.getContentType());
		httpPost.setEntity(reqEntity);
		return this.execute(httpPost);
	}
	
	
	/**
	 * 上传文件到基础服务
	 * @author yangsheng@angeldoctor 
	 * @return
	 */
	public String uploadMultiFileToCommentService(){
		if(fileList ==null || fileList.size() <=0){
			throw new NullPointerException("fileList is null");
		}
		HttpPost httpPost = this.getPost();
		MultipartEntityBuilder builer = MultipartEntityBuilder.create();
		for(File file :fileList){
			FileBody fileBody = new FileBody(file);
			builer.addPart("file",fileBody);
		}
		HttpEntity reqEntity =builer.build();
		httpPost.addHeader(reqEntity.getContentType());
		httpPost.setEntity(reqEntity);
		return this.execute(httpPost);
	}
	
	/**
	 * 上传文件到基础服务
	 * @author yangsheng@angeldoctor 
	 * @return
	 */
	public String uploadFileToCommentService(){
		if(file ==null){
			throw new NullPointerException("file is null");
		}
		HttpPost httpPost = this.getPost();
		MultipartEntityBuilder builer = MultipartEntityBuilder.create();
		FileBody fileBody = new FileBody(file);
		builer.addPart("file",fileBody);
		HttpEntity reqEntity =builer.build();
		httpPost.addHeader(reqEntity.getContentType());
		httpPost.setEntity(reqEntity);
		return this.execute(httpPost);
	}
	
	/**
	 * 执行请求
	 * @author yangsheng@angeldoctor 
	 * @param httpRequest
	 * @return
	 */
	private String execute(HttpRequestBase httpRequest){
		CloseableHttpClient httpclient = this.getHttpClient();
		CloseableHttpResponse response =null;
		try {
			response = httpclient.execute(httpRequest);
			log.info(response.getStatusLine());
		    HttpEntity entity = response.getEntity();
		    result =EntityUtils.toString(entity, "utf-8");
		    EntityUtils.consume(entity);
		} catch (ClientProtocolException e) {
			log.error("HttpClientTools.execute.error",e);
		} catch (IOException e) {
			log.error("HttpClientTools.execute.error",e);
		} finally {
			releaseConnection(httpclient,response);
		}
		return result;
	}
	
	private HttpPost getPost(){
		RequestConfig requestConfig = this.getReqConfig();
		HttpPost httpPost = new HttpPost(url);
		httpPost.setConfig(requestConfig);
		return httpPost;
	}
	
	private CloseableHttpClient getHttpClient(){
		CloseableHttpClient httpclient = HttpClients.createDefault();
		return httpclient;
	}
	
	private RequestConfig getReqConfig(){
		RequestConfig requestConfig = RequestConfig.custom()
				.setConnectionRequestTimeout(connectionRequestTimeout)
		        .setConnectTimeout(connectTimeout)
		        .setSocketTimeout(socketTimeout)
		        .build();
		return requestConfig;
	}
	
	private  void releaseConnection(CloseableHttpClient httpclient,CloseableHttpResponse response){
		try {
			if(response !=null){
				response.close();
			}
			if(httpclient !=null){
				httpclient.close();
			}
		} catch (IOException e) {
			 log.error("HttpClientTools.realConnection.error",e);
		}
	}
	
	
}
